@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: chanserv.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: chanserv.c 1061 2007-06-02 16:40:56Z tjfontaine $
  */
 
 #include "stdinc.h"
@@ -260,7 +260,7 @@ static struct ServiceMessage unforbid_msgtab = {
   CS_HELP_UNFORBID_SHORT, CS_HELP_UNFORBID_LONG, m_unforbid
 };
 
-INIT_MODULE(chanserv, "$Revision: 1043 $")
+INIT_MODULE(chanserv, "$Revision: 1061 $")
 {
   chanserv = make_service("ChanServ");
   clear_serv_tree_parse(&chanserv->msg_tree);
@@ -488,6 +488,7 @@ m_info(struct Service *service, struct Client *client,
   void *first, *listptr;  
   char buf[IRC_BUFSIZE+1] = {0};
   char *nick;
+  struct ChanAccess *access = NULL;
 
   if(db_is_chan_forbid(parv[1]))
   {
@@ -496,15 +497,29 @@ m_info(struct Service *service, struct Client *client,
   }
   
   regchptr = db_find_chan(parv[1]);
+
+  if(client->nickname)
+    access = db_find_chanaccess(regchptr->id, client->nickname->id);
   
   reply_user(service, service, client, CS_INFO_CHAN_START, regchptr->channel);
   reply_time(service, client, CS_INFO_REGTIME_FULL, regchptr->regtime);
-  reply_user(service, service, client, CS_INFO_CHAN, regchptr->description, 
-      regchptr->url == NULL ? "Not Set" : regchptr->url, 
-      regchptr->email == NULL ? "Not Set" : regchptr->email, 
-      regchptr->topic == NULL ? "Not Set" : regchptr->topic, 
-      regchptr->entrymsg == NULL ? "Not Set" : regchptr->entrymsg,
-      regchptr->mlock == NULL ? "Not Set" : regchptr->mlock);
+  if(IsOper(client) || (access != NULL && access->level >= MEMBER_FLAG))
+  {
+    reply_user(service, service, client, CS_INFO_CHAN_MEMBER,
+        regchptr->description, 
+        regchptr->url == NULL ? "Not Set" : regchptr->url, 
+        regchptr->email == NULL ? "Not Set" : regchptr->email, 
+        regchptr->topic == NULL ? "Not Set" : regchptr->topic, 
+        regchptr->entrymsg == NULL ? "Not Set" : regchptr->entrymsg,
+        regchptr->mlock == NULL ? "Not Set" : regchptr->mlock);
+  }
+  else
+  {
+    reply_user(service, service, client, CS_INFO_CHAN_NMEMBER,
+        regchptr->description, 
+        regchptr->url == NULL ? "Not Set" : regchptr->url, 
+        regchptr->email == NULL ? "Not Set" : regchptr->email);
+  }
 
   if((listptr = db_list_first(CHMASTER_LIST, regchptr->id, 
           (void**)&nick)) != NULL)
@@ -553,6 +568,9 @@ m_info(struct Service *service, struct Client *client,
   reply_user(service, service, client, CS_INFO_OPTION, "LEAVEOPS",
       regchptr->leaveops ? "ON" : "OFF");
 
+  if(access != NULL)
+    MyFree(access);
+
   free_regchan(regchptr);
 }
 
@@ -618,11 +636,12 @@ m_access_add(struct Service *service, struct Client *client,
           regchptr->channel);
       if(chptr == NULL)
         free_regchan(regchptr);
-      free_chanaccess(oldaccess);
+      MyFree(oldaccess);
+      MyFree(access);
       return;
     }
     db_list_del_index(DELETE_CHAN_ACCESS, access->account, access->channel);
-    free_chanaccess(oldaccess);
+    MyFree(oldaccess);
   }
  
   if(db_list_add(CHACCESS_LIST, access))
@@ -639,7 +658,7 @@ m_access_add(struct Service *service, struct Client *client,
   if (chptr == NULL)
     free_regchan(regchptr);
 
-  free_chanaccess(access);
+  MyFree(access);
 }
 
 
@@ -678,10 +697,11 @@ m_access_del(struct Service *service, struct Client *client,
         regchptr->channel);
     if(chptr == NULL)
       free_regchan(regchptr);
-    free_chanaccess(access);
+    MyFree(access);
     return;
   }
-  free_chanaccess(access);
+
+  MyFree(access);
 
   if(db_list_del_index(DELETE_CHAN_ACCESS, nickid, regchptr->id))
   {
@@ -714,7 +734,7 @@ m_access_list(struct Service *service, struct Client *client,
   if (handle == NULL)
   {
     reply_user(service, service, client, CS_ACCESS_LISTEND, regchptr->channel);
-    free_chanaccess(access);
+    MyFree(access);
     if(chptr == NULL)
       free_regchan(regchptr);
     return;
@@ -744,12 +764,12 @@ m_access_list(struct Service *service, struct Client *client,
     nick = db_get_nickname_from_id(access->account);
     reply_user(service, service, client, CS_ACCESS_LIST, i++, nick, level);
 
-    free_chanaccess(access);
+    MyFree(access);
     MyFree(nick);
     handle = db_list_next(handle, CHACCESS_LIST, (void **)&access);
   }
 
-  free_chanaccess(access);
+  MyFree(access);
   db_list_done(first);
   reply_user(service, service, client, CS_ACCESS_LISTEND, regchptr->channel);
 
@@ -1441,13 +1461,13 @@ m_voice(struct Service *service, struct Client *client, int parc, char *parv[])
   else if(access->level < CHANOP_FLAG)
   {
     reply_user(service, service, client, CS_NO_VOICE_OTHERS, parv[1]);
-    free_chanaccess(access);
+    MyFree(access);
     return;
   }
   else
     target = find_client(parv[2]);
 
-  free_chanaccess(access);
+  MyFree(access);
 
   if(target == NULL || (ms = find_channel_link(target, chptr)) == NULL)
   {
@@ -1489,13 +1509,13 @@ m_devoice(struct Service *service, struct Client *client, int parc, char *parv[]
   else if(access->level < CHANOP_FLAG)
   {
     reply_user(service, service, client, CS_NO_DEVOICE_OTHERS, parv[1]);
-    free_chanaccess(access);
+    MyFree(access);
     return;
   }
   else
     target = find_client(parv[2]);
 
-  free_chanaccess(access);
+  MyFree(access);
 
   if(target == NULL || (ms = find_channel_link(target, chptr)) == NULL)
   {
@@ -1680,6 +1700,7 @@ m_set_flag(struct Service *service, struct Client *client,
         break;
       case SET_CHAN_FLOODSERV:
         on = regchptr->floodserv;
+        break;
       default:
         on = FALSE;
         break;
@@ -2000,8 +2021,7 @@ cs_on_client_join(va_list args)
     else
       level = access->level;
 
-    if(access != NULL)
-      free_chanaccess(access);
+    MyFree(access);
   }
 
   if(regchptr->restricted && level < MEMBER_FLAG)
