@@ -61,7 +61,7 @@ static struct ServiceMessage help_msgtab = {
   FS_HELP_LONG, m_help
 };
 
-INIT_MODULE(floodserv, "$Revision: 1043 $")
+INIT_MODULE(floodserv, "$Revision: 1072 $")
 {
   dlink_node *ptr = NULL, *next_ptr = NULL;
   int i;
@@ -321,12 +321,18 @@ fs_on_client_part(va_list args)
   {
     ilog(L_DEBUG, "FloodServ removed from %s by %s", channel->chname,
       source->name);
-    if(channel->regchan != NULL && channel->regchan->flood_hash != NULL)
+    /* TODO The channel may already be destroyed we need to find out why */
+    if(channel != NULL && channel->regchan != NULL
+      && channel->regchan->flood_hash != NULL)
+    {
       floodserv_free_channel(channel->regchan);
+    }
   }
   else
   {
-    if(channel->regchan != NULL && channel->regchan->flood_hash != NULL)
+    /* TODO The channel may already be destroyed we need to find out why */
+    if(channel != NULL && channel->regchan != NULL
+      && channel->regchan->flood_hash != NULL)
     {
       if(channel->members.length == 2 && IsMember(fsclient, channel))
         part_channel(fsclient, channel->chname, "");
@@ -406,10 +412,19 @@ fs_on_privmsg(va_list args)
         ilog(L_NOTICE, "Flood %s@%s TRIGGERED NETWORK MSG FLOOD Message: %s",
           source->name, host, message);
         snprintf(mask, IRC_BUFSIZE, "*@%s", host);
+
+        if((akill = db_find_akill(mask)) != NULL)
+        {
+          ilog(L_NOTICE, "Flood AKILL Already Exists");
+          free_serviceban(akill);
+          return pass_callback(fs_privmsg_hook, source, channel, message);
+        }
+
         akill = akill_add(floodserv, fsclient, mask, FS_KILL_MSG, FS_KILL_DUR);
 
         if(akill != NULL)
           free_serviceban(akill);
+
         return pass_callback(fs_privmsg_hook, source, channel, message);
         break;
     }
