@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: irc.c 921 2007-05-03 21:05:33Z stu $
+ *  $Id: irc.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 #include "stdinc.h"
@@ -38,6 +38,7 @@ static void m_topic(struct Client *, struct Client *, int, char*[]);
 static void m_kill(struct Client *, struct Client *, int, char*[]);
 static void m_kick(struct Client *, struct Client *, int, char*[]);
 static void m_version(struct Client *, struct Client *, int, char*[]);
+static void m_tburst(struct Client *, struct Client *, int, char*[]);
 
 //static void do_user_modes(struct Client *client, const char *modes);
 
@@ -190,6 +191,11 @@ static struct Message notice_msgtab = {
   { m_notice, m_ignore }
 };
 
+static struct Message tburst_msgtab = {
+  "TBURST", 0, 0, 4, 0, 0, 0,
+  { m_tburst, m_tburst }
+};
+
 static dlink_node *connected_hook;
 static dlink_node *newuser_hook;
 static dlink_node *privmsg_hook;
@@ -218,7 +224,7 @@ struct ModeList ModeList[] = {
   { 0, '\0' }
 };
 
-INIT_MODULE(irc, "$Revision: 921 $")
+INIT_MODULE(irc, "$Revision: 1043 $")
 {
   connected_hook  = install_hook(connected_cb, irc_server_connected);
   newuser_hook    = install_hook(send_newuser_cb, irc_sendmsg_nick);
@@ -262,6 +268,7 @@ INIT_MODULE(irc, "$Revision: 921 $")
   mod_add_cmd(&kill_msgtab);
   mod_add_cmd(&kick_msgtab);
   mod_add_cmd(&notice_msgtab);
+  mod_add_cmd(&tburst_msgtab);
 }
 
 CLEANUP_MODULE
@@ -289,6 +296,10 @@ CLEANUP_MODULE
   mod_del_cmd(&trace_msgtab);
   mod_del_cmd(&stats_msgtab);
   mod_del_cmd(&topic_msgtab);
+  mod_del_cmd(&tburst_msgtab);
+  mod_del_cmd(&notice_msgtab);
+  mod_del_cmd(&kick_msgtab);
+  mod_del_cmd(&kill_msgtab);
 }
 
 /** Introduce a new server; currently only useful for connect and jupes
@@ -474,8 +485,16 @@ irc_sendmsg_topic(va_list args)
 
   source = source;
 
-  sendto_server(uplink, ":%s TBURST 1 %s %lu %s :%s", me.name, chptr->chname,
-      CurrentTime, setter->name, topic);
+  if(topic == NULL)
+  {
+    sendto_server(uplink, ":%s TBURST 1 %s %lu %s", me.name, chptr->chname,
+        CurrentTime, setter->name);
+  }
+  else
+  {
+    sendto_server(uplink, ":%s TBURST 1 %s %lu %s :%s", me.name, chptr->chname,
+        CurrentTime, setter->name, topic);
+  }
 
   return NULL;
 }
@@ -831,6 +850,9 @@ m_sjoin(struct Client *client, struct Client *source, int parc, char *parv[])
       case 'p':
         mode.mode |= MODE_PARANOID;
         break;
+      case 'c':
+        mode.mode |= MODE_NOCOLOR;
+        break;
       case 'k':
         strlcpy(mode.key, parv[4 + args], sizeof(mode.key));
         args++;
@@ -1118,7 +1140,7 @@ m_kick(struct Client *client, struct Client *source, int parc, char *parv[])
 
   reason = (EmptyString(parv[3])) ? parv[2] : parv[3];
 
-  target = find_client(parv[2]);
+  target = find_person(source, parv[2]);
   if(target == NULL)
     return;
 
@@ -1139,12 +1161,18 @@ m_quit(struct Client *client, struct Client *source, int parc, char *parv[])
 static void
 m_kill(struct Client *client, struct Client *source, int parc, char *parv[])
 {
-  char *comment = (parc > 1 && parv[1]) ? parv[1] : client->name;
+  struct Client *target = find_person(source, parv[1]);
+  char *comment;
+  
+  if(target == NULL)
+    return;
+  
+  comment = (parc > 1 && parv[2]) ? parv[2] : target->name;
 
   if (strlen(comment) > (size_t)KICKLEN)
     comment[KICKLEN] = '\0';
 
-  exit_client(source, source, comment);
+  exit_client(target, source, comment);
 }
 
 static void
@@ -1275,6 +1303,34 @@ m_version(struct Client *client_p, struct Client *source_p,
   execute_callback(send_nosuchsrv_cb, source_p->name, parv[1]);
 }
 
+static void
+m_tburst(struct Client *client_p, struct Client *source_p, int parc, 
+    char *parv[])
+{
+  struct Channel *chptr = NULL;
+  time_t remote_topic_ts = atol(parv[3]);
+  const char *topic = "";
+  const char *setby = "";
+
+  /*
+   * Do NOT test parv[5] for an empty string and return if true!
+   * parv[5] CAN be an empty string, i.e. if the other side wants
+   * to unset our topic.  Don't forget: an empty topic is also a
+   * valid topic.
+   */
+
+  if ((chptr = hash_find_channel(parv[2])) == NULL)
+    return;
+
+  if (parc == 6)
+  {
+    topic = parv[5];
+    setby = parv[4];
+  }
+
+  set_channel_topic(chptr, topic, setby, remote_topic_ts);
+}
+
 static void *
 irc_sendmsg_nosuchsrv(va_list args)
 {
