@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: nickserv.c 917 2007-05-03 19:30:58Z stu $
+ *  $Id: nickserv.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 #include "stdinc.h"
@@ -51,7 +51,6 @@ static void m_drop(struct Service *, struct Client *, int, char *[]);
 static void m_help(struct Service *, struct Client *, int, char *[]);
 static void m_identify(struct Service *, struct Client *, int, char *[]);
 static void m_register(struct Service *, struct Client *, int, char *[]);
-static void m_ghost(struct Service *, struct Client *, int, char *[]);
 static void m_link(struct Service *, struct Client *, int, char *[]);
 static void m_unlink(struct Service *, struct Client *, int, char *[]);
 static void m_info(struct Service *,struct Client *, int, char *[]);
@@ -150,8 +149,8 @@ static struct ServiceMessage access_msgtab = {
 };
 
 static struct ServiceMessage ghost_msgtab = {
-  NULL, "GHOST", 0, 1, 2, 0, USER_FLAG, NS_HELP_GHOST_SHORT, NS_HELP_GHOST_LONG,
-  m_ghost
+  NULL, "GHOST", 0, 2, 2, SFLG_ALIAS, USER_FLAG, NS_HELP_REGAIN_SHORT, 
+  NS_HELP_REGAIN_LONG, m_regain
 };
 
 static struct ServiceMessage link_msgtab = {
@@ -199,7 +198,7 @@ static struct ServiceMessage list_msgtab = {
   NS_HELP_LIST_LONG, m_list
 };
 
-INIT_MODULE(nickserv, "$Revision: 917 $")
+INIT_MODULE(nickserv, "$Revision: 1043 $")
 {
   nickserv = make_service("NickServ");
   clear_serv_tree_parse(&nickserv->msg_tree);
@@ -306,6 +305,44 @@ process_release_list(void *param)
   }
 }
 
+static void
+handle_nick_change(struct Service *service, struct Client *client, 
+    const char *name, unsigned int message)
+{
+  struct Client *target;
+
+  if((target = find_client(name)) != NULL)
+  {
+    if(MyConnect(target))
+    {
+      dlinkFindDelete(&nick_release_list, target);
+      exit_client(target, &me, "Enforcer no longer needed");
+      reply_user(service, service, client, message, name);
+      send_nick_change(service, client, name);
+    }
+    else if(target != client)
+    {
+      target->release_to = client;
+      strlcpy(target->release_name, name, NICKLEN);
+      guest_user(target);
+      reply_user(service, service, client, message, name);
+    }
+    else
+    {
+      identify_user(client);
+      db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
+      reply_user(service, service, client, message, name);
+    }
+  }
+  else
+  {
+    reply_user(service, service, client, message, name);
+    send_nick_change(service, client, name);
+  }
+
+  SetIdentified(client);
+}
+
 static void 
 m_register(struct Service *service, struct Client *client, 
     int parc, char *parv[])
@@ -334,8 +371,7 @@ m_register(struct Service *service, struct Client *client,
 
   if(client->nickname != NULL)
   {
-    reply_user(service, service, client, NS_ALREADY_REG, 
-        client->nickname->real_nick);
+    reply_user(service, service, client, NS_ALREADY_REG, client->name);
     return;
   }
     
@@ -361,6 +397,7 @@ m_register(struct Service *service, struct Client *client,
   {
     client->nickname = nick;
     identify_user(client);
+    db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
 
     reply_user(service, service, client, NS_REG_COMPLETE, client->name);
     global_notice(NULL, "%s!%s@%s registered nick %s\n", client->name, 
@@ -376,18 +413,28 @@ m_drop(struct Service *service, struct Client *client,
         int parc, char *parv[])
 {
   struct Client *target;
+  struct Nick *nick = db_find_nick(client->name);
+  char *target_nick = NULL;
+
+  assert(nick != NULL);
 
   /* This might be being executed via sudo, find the real user of the nick */
-  if(irccmp(client->name, client->nickname->nick) != 0 && 
-      irccmp(client->name, client->nickname->real_nick) != 0)
-    target = find_client(client->nickname->real_nick);
+  if(client->nickname->id != nick->id)
+  {
+    target_nick = db_get_nickname_from_nickid(client->nickname->nickid);
+    target = find_client(target_nick);
+  }
   else
     target = client;
 
+  free_nick(nick);
+
   if(target == client)
   {
+    /* A normal user dropping their own nick */
     if(parc == 0)
     {
+      /* No auth code, so give them one and do nothing else*/
       char buf[IRC_BUFSIZE+1] = {0};
       char *hmac;
 
@@ -403,6 +450,7 @@ m_drop(struct Service *service, struct Client *client,
     }
     else
     {
+      /* Auth code given, check it and do the drop */
       char buf[IRC_BUFSIZE+1] = {0};
       char *hmac;
       char *auth;
@@ -438,8 +486,12 @@ m_drop(struct Service *service, struct Client *client,
     }
   }
 
-  if(db_delete_nick(client->nickname->id, client->nickname->nickid, 
-        client->nickname->nick)) 
+  /* Authentication passed(possibly because they're using sudo), go ahead and
+   * drop
+   */
+
+  if(db_delete_nick(client->nickname->id, client->nickname->nickid,
+        client->nickname->pri_nickid)) 
   {
     if(target != NULL)
     {
@@ -450,34 +502,33 @@ m_drop(struct Service *service, struct Client *client,
       target->access = USER_FLAG;
       send_umode(nickserv, target, "-R");
       reply_user(service, service, client, NS_NICK_DROPPED, target->name);
-      global_notice(NULL, "%s!%s@%s dropped nick %s\n", client->name, 
+      ilog(L_NOTICE, "%s!%s@%s dropped nick %s", client->name, 
         client->username, client->host, target->name);
     }
     else
     {
-      reply_user(service, service, client, NS_NICK_DROPPED,
-          client->nickname->real_nick);
-      global_notice(NULL, "%s!%s@%s dropped nick %s\n", client->name, 
-        client->username, client->host, client->nickname->real_nick);
+      reply_user(service, service, client, NS_NICK_DROPPED, target_nick);
+      ilog(L_NOTICE, "%s!%s@%s dropped nick %s", client->name, 
+        client->username, client->host, target_nick);
     }
   }
   else
   {
     if(target == NULL)
     {
-      global_notice(NULL, "Error: %s!%s@%s could not DROP nick %s\n", 
-          client->name, client->username, client->host, 
-          client->nickname->real_nick);
-      reply_user(service, service, client, NS_NICK_DROPFAIL, 
-          client->nickname->real_nick);
+      ilog(L_NOTICE, "Error: %s!%s@%s could not DROP nick %s", 
+          client->name, client->username, client->host, target_nick);
+      reply_user(service, service, client, NS_NICK_DROPFAIL, target_nick);
     }
     else
     {
-      global_notice(NULL, "Error: %s!%s@%s could not DROP nick %s\n", 
+      ilog(L_NOTICE, "Error: %s!%s@%s could not DROP nick %s", 
           client->name, client->username, client->host, target->name);
       reply_user(service, service, client, NS_NICK_DROPFAIL, target->name);
     }
   }
+
+  MyFree(target_nick);
 }
 
 static void
@@ -510,34 +561,20 @@ m_identify(struct Service *service, struct Client *client,
     return;
   }
 
+  if(client->nickname != NULL)
+    free_nick(client->nickname);
+
   client->nickname = nick;
   dlinkFindDelete(&nick_enforce_list, client);
 
   if(parc > 1)
+    handle_nick_change(service, client, name, NS_IDENTIFIED);
+  else
   {
-    struct Client *target;
-    /* User specified nick to change them too, so do so */
-    if((target = find_client(name)) != NULL)
-    {
-      if(MyConnect(target))
-      {
-        exit_client(target, &me, "Enforcer no longer needed");
-        send_nick_change(service, client, name);
-      }
-      else if(target != client)
-      {
-        target->release_to = client;
-        strlcpy(target->release_name, name, NICKLEN);
-        guest_user(target);
-      }
-    }
-    else
-    {
-      send_nick_change(service, client, name);
-    }
+    identify_user(client);
+    db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
+    reply_user(service, service, client, NS_IDENTIFIED, name);
   }
-  identify_user(client);
-  reply_user(service, service, client, NS_IDENTIFIED, name);
 }
 
 static void
@@ -579,7 +616,7 @@ static void
 m_set_language(struct Service *service, struct Client *client,
         int parc, char *parv[])
 {
-  /* No paramters, list languages */
+  /* No parameters, list languages */
   if(parc == 0)
   {
     int i;
@@ -921,7 +958,7 @@ m_access_list(struct Service *service, struct Client *client, int parc,
     char *parv[])
 {
   struct Nick *nick;
-  struct AccessEntry *entry;
+  struct AccessEntry *entry = NULL;
   void *first, *listptr;
   int i = 1;
 
@@ -931,6 +968,7 @@ m_access_list(struct Service *service, struct Client *client, int parc,
  
   if((listptr = db_list_first(ACCESS_LIST, nick->id, (void**)&entry)) == NULL)
   {
+    MyFree(entry);
     return;
   }
 
@@ -943,6 +981,7 @@ m_access_list(struct Service *service, struct Client *client, int parc,
     listptr = db_list_next(listptr, ACCESS_LIST, (void**)&entry);
   }
 
+  MyFree(entry);
   db_list_done(first);
 }
 
@@ -963,14 +1002,13 @@ m_access_del(struct Service *service, struct Client *client, int parc,
 }
 
 static void
-m_ghost(struct Service *service, struct Client *client, int parc, char *parv[])
+m_regain(struct Service *service, struct Client *client, int parc, char *parv[])
 {
   struct Nick *nick;
-  struct Client *target;
 
   if(find_client(parv[1]) == NULL)
   {
-    reply_user(service, service, client, NS_GHOST_NOTONLINE, parv[1]);
+    reply_user(service, service, client, NS_REGAIN_NOTONLINE, parv[1]);
     return;
   }
 
@@ -980,40 +1018,26 @@ m_ghost(struct Service *service, struct Client *client, int parc, char *parv[])
     return;
   }
 
-  if(ircncmp(client->name, parv[1], NICKLEN) == 0)
-  {
-    free_nick(nick);
-    reply_user(service, service, client, NS_GHOST_NOSELF, parv[1]);
-    return;
-  }
-
   if(parc == 1 && (nick->secure || !IsOnAccess(client)))
   {
     free_nick(nick);
-    reply_user(service, service, client, NS_GHOST_FAILED_SECURITY, parv[1]);   
+    reply_user(service, service, client, NS_REGAIN_FAILED_SECURITY, parv[1]);   
     return;
   }
 
   if((parc == 2 && !check_nick_pass(nick, parv[2])))
   {
     free_nick(nick);
-    reply_user(service, service, client, NS_GHOST_FAILED, parv[1]);   
+    reply_user(service, service, client, NS_REGAIN_FAILED, parv[1]);   
     return;
   }
+  
+  if(client->nickname != NULL)
+    free_nick(client->nickname);
 
-  if((target = find_client(parv[1])) != NULL)
-  {
-    if(MyConnect(target))
-    {
-      exit_client(target, &me, "Enforcer no longer needed");
-      send_nick_change(service, client, parv[1]);
-    }
-    else
-    {
-      guest_user(target);
-    }
-  }
-  reply_user(service, service, client, NS_GHOST_SUCCESS, parv[1]);
+  client->nickname = nick;
+
+  handle_nick_change(service, client, parv[1], NS_REGAIN_SUCCESS);
 }
 
 static void
@@ -1049,13 +1073,13 @@ m_link(struct Service *service, struct Client *client, int parc, char *parv[])
     return;
   }
 
-  strlcpy(master_nick->nick, nick->nick, sizeof(master_nick->nick));
-  master_nick->nickid = nick->nickid;
-  master_nick->pri_nickid = master_nick->nickid;
-  client->nickname = master_nick;
   reply_user(service, service, client, NS_LINK_OK, nick->nick, parv[1]);
+  
+  free_nick(master_nick);
   free_nick(nick);
-  nick = NULL;
+
+  client->nickname = db_find_nick(client->name);
+  assert(client->nickname != NULL);
 }
 
 static void
@@ -1079,7 +1103,7 @@ m_info(struct Service *service, struct Client *client, int parc, char *parv[])
 {
   struct Nick *nick;
   struct Client *target;
-  struct InfoChanList *chan;
+  struct InfoChanList *chan = NULL;
   char *name;
   char *link;
   char buf[IRC_BUFSIZE+1] = {0};
@@ -1122,6 +1146,9 @@ m_info(struct Service *service, struct Client *client, int parc, char *parv[])
     name = parv[1];
   }
 
+  reply_user(service, service, client, NS_INFO_START, name, 
+      nick->last_realname != NULL ? nick->last_realname : "Unknown");
+
   if((listptr = db_list_first(NICKLINK_LIST, nick->id, (void**)&link)) != NULL)
   {
     int comma = 0;
@@ -1185,6 +1212,14 @@ m_info(struct Service *service, struct Client *client, int parc, char *parv[])
     if(*buf != '\0')
       reply_user(service, service, client, NS_INFO_LINKS, buf);
 
+    if(nick->nickid != nick->pri_nickid)
+    {
+      char *prinick = db_get_nickname_from_id(nick->id);
+
+      reply_user(service, service, client, NS_INFO_MASTER, prinick);
+      MyFree(prinick);
+    }
+
     if((listptr = db_list_first(NICKCHAN_LIST, nick->id, (void**)&chan)) != NULL)
     {
       first = listptr;
@@ -1196,8 +1231,11 @@ m_info(struct Service *service, struct Client *client, int parc, char *parv[])
         MyFree(chan);
         listptr = db_list_next(listptr, NICKCHAN_LIST, (void**)&chan);
       }
+      MyFree(chan);
       db_list_done(first);
     }
+    else
+      MyFree(chan);
   }
   else if(!nick->priv)
     reply_user(service, service, client, NS_INFO_EMAIL, nick->email);
@@ -1308,53 +1346,6 @@ m_unforbid(struct Service *service, struct Client *client, int parc,
   }
 }
 
-static void
-m_regain(struct Service *service, struct Client *client, int parc, 
-    char *parv[])
-{
-  struct Nick *nick;
-  struct Client *enforcer;
-
-  enforcer = find_client(parv[1]);
-  
-  if((nick = db_find_nick(parv[1])) == NULL)
-  {
-    reply_user(service, service, client, NS_REG_FIRST, parv[1]);
-    return;
-  }
-
-  if(!check_nick_pass(nick, parv[2]))
-  {
-    free_nick(nick);
-    reply_user(service, service, client, NS_REGAIN_FAIL, parv[1]);
-    return;
-  }
-
-  if(client->nickname)
-    free_nick(client->nickname);
-  client->nickname = nick;
- 
-  if(enforcer != NULL && dlinkFind(&nick_release_list, enforcer) != NULL)
-  {
-    dlinkFindDelete(&nick_enforce_list, client);
-    exit_client(enforcer, &me, "RELEASE command issued");
-    send_nick_change(service, client, parv[1]);
-  }
-  else if(enforcer != NULL)
-  {
-    enforcer->release_to = client;
-    strlcpy(enforcer->release_name, parv[1], NICKLEN);
-    guest_user(enforcer);
-  }
-  else
-  {
-    send_nick_change(service, client, parv[1]);
-  }
- 
-  identify_user(client);
-  reply_user(service, service, client, NS_REGAINED, client->name);
-}
-
 static void 
 m_sudo(struct Service *service, struct Client *client, int parc, char *parv[])
 {
@@ -1602,6 +1593,7 @@ ns_on_nick_change(va_list args)
     send_nick_change(nickserv, client, user->release_name);
     user->release_to = NULL;
     memset(user->release_name, 0, sizeof(user->release_name));
+    db_set_string(SET_NICK_LAST_REALNAME, client->nickname->id, client->info);
     identify_user(client);
   }
 
@@ -1637,7 +1629,10 @@ ns_on_nick_change(va_list args)
   {
     ilog(L_DEBUG, "Nick Change: %s->%s(nick not registered)", oldnick, user->name);
     if(user->nickname != NULL)
+    {
       free_nick(user->nickname);
+      user->nickname = NULL;
+    }
     return pass_callback(ns_nick_hook, user, oldnick);
   }
 
@@ -1647,6 +1642,7 @@ ns_on_nick_change(va_list args)
     if(user->nickname != NULL)
       free_nick(user->nickname);
     user->nickname = nick_p;
+    db_set_string(SET_NICK_LAST_REALNAME, user->nickname->id, user->info);
     identify_user(user);
     return pass_callback(ns_nick_hook, user, oldnick);
   }
@@ -1663,6 +1659,8 @@ ns_on_nick_change(va_list args)
         free_nick(user->nickname);
       user->nickname = nick_p;
       identify_user(user);
+      db_set_string(SET_NICK_LAST_REALNAME, user->nickname->id, user->info);
+      reply_user(nickserv, nickserv, user, NS_IDENTIFY_ACCESS, user->name);
     }
   }
   else
@@ -1680,6 +1678,9 @@ ns_on_nick_change(va_list args)
  
     ilog(L_DEBUG, "%s changed nick to %s(no access entry)", oldnick, user->name);
   }
+
+  if(nick_p != user->nickname)
+    free_nick(nick_p);
   
   return pass_callback(ns_nick_hook, user, oldnick);
 }
@@ -1716,6 +1717,7 @@ ns_on_newuser(va_list args)
       free_nick(newuser->nickname);
     newuser->nickname = nick_p;
     identify_user(newuser);
+    db_set_string(SET_NICK_LAST_REALNAME, newuser->nickname->id, newuser->info);
     return pass_callback(ns_newuser_hook, newuser);
   }
 
@@ -1730,6 +1732,8 @@ ns_on_newuser(va_list args)
         free_nick(newuser->nickname);
       newuser->nickname = nick_p;
       identify_user(newuser);
+      db_set_string(SET_NICK_LAST_REALNAME, newuser->nickname->id, newuser->info);
+      reply_user(nickserv, nickserv, newuser, NS_IDENTIFY_ACCESS, newuser->name);
     }
   }
   else
@@ -1746,6 +1750,9 @@ ns_on_newuser(va_list args)
     }
     ilog(L_DEBUG, "new user:%s(no access entry)", newuser->name);
   }
+
+  if(nick_p != newuser->nickname)
+    free_nick(nick_p);
   
   return pass_callback(ns_newuser_hook, newuser);
 }
