@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: operserv.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: operserv.c 1086 2007-06-26 14:06:21Z swalsh $
  */
 
 #include "stdinc.h"
@@ -49,7 +49,7 @@ static void m_akill_list(struct Service *, struct Client *, int, char *[]);
 static void m_akill_del(struct Service *, struct Client *, int, char *[]);
 
 static struct ServiceMessage help_msgtab = {
-  NULL, "HELP", 0, 0, 2, 0, ADMIN_FLAG, OS_HELP_SHORT, OS_HELP_LONG, m_help
+  NULL, "HELP", 0, 0, 2, 0, OPER_FLAG, OS_HELP_SHORT, OS_HELP_LONG, m_help
 };
 
 static struct ServiceMessage mod_subs[] = {
@@ -99,7 +99,7 @@ static struct ServiceMessage akill_subs[] = {
 };
 
 static struct ServiceMessage akill_msgtab = {
-  akill_subs, "AKILL", 0, 2, 2, 0, ADMIN_FLAG, OS_AKILL_HELP_SHORT, 
+  akill_subs, "AKILL", 0, 2, 2, 0, OPER_FLAG, OS_AKILL_HELP_SHORT, 
   OS_AKILL_HELP_LONG, NULL
 };
 
@@ -108,7 +108,7 @@ static struct ServiceMessage set_msgtab = {
   m_operserv_notimp
 };
 
-INIT_MODULE(operserv, "$Revision: 1043 $")
+INIT_MODULE(operserv, "$Revision: 1086 $")
 {
   operserv = make_service("OperServ");
   clear_serv_tree_parse(&operserv->msg_tree);
@@ -184,16 +184,16 @@ m_mod_load(struct Service *service, struct Client *client,
     return;
   }
 
-  global_notice(service, "Loading %s by request of %s",
+  ilog(L_NOTICE, "Loading %s by request of %s",
       parm, client->name);
   if (load_module(parm) == 1)
   {
-    global_notice(service, "Module %s loaded", parm);
+    ilog(L_NOTICE, "Module %s loaded", parm);
     reply_user(service, service, client, OS_MOD_LOADED, parm);
   }
   else
   {
-    global_notice(service, "Module %s could not be loaded!", parm);
+    ilog(L_NOTICE, "Module %s could not be loaded!", parm);
     reply_user(service, service, client, OS_MOD_LOADFAIL, parm);
   }
 }
@@ -208,25 +208,31 @@ m_mod_reload(struct Service *service, struct Client *client,
 
   mbn = basename(parm);
   module = find_module(mbn, 0);
+  if(irccmp(module->name, service->name) == 0)
+  {
+    ilog(L_NOTICE, "%s tried to reload %s.  Can't be done because it's me!",
+        client->name, service->name);
+    reply_user(service, service, client, OS_MOD_CANTRELOAD, parm);
+    return;
+  }
   if (module == NULL)
   {
-    global_notice(service,
-        "Module %s reload requested by %s, but failed because not loaded",
+    ilog(L_NOTICE, "Module %s reload requested by %s, but failed because not loaded",
         parm, client->name);
     reply_user(service, service, client, OS_MOD_NOTLOADED, parm, client->name);
     return;
   }
-  global_notice(service, "Reloading %s by request of %s", parm, client->name);
+  ilog(L_NOTICE, "Reloading %s by request of %s", parm, client->name);
   reply_user(service, service, client, OS_MOD_RELOADING, parm, client->name);
   unload_module(module);
   if (load_module(parm) == 1)
   {
-    global_notice(service, "Module %s loaded", parm);
+    ilog(L_NOTICE, "Module %s loaded", parm);
     reply_user(service, service, client, OS_MOD_LOADED,parm);
   }
   else
   {
-    global_notice(service, "Module %s could not be loaded!", parm);
+    ilog(L_NOTICE, "Module %s could not be loaded!", parm);
     reply_user(service, service, client, OS_MOD_LOADFAIL, parm);
   }
 }
