@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: client.c 888 2007-04-30 22:39:01Z tjfontaine $
+ *  $Id: client.c 1043 2007-05-18 09:24:56Z swalsh $
  */
 
 
@@ -116,6 +116,12 @@ init_client()
   client_heap = BlockHeapCreate("client", sizeof(struct Client), CLIENT_HEAP_SIZE);
 }
 
+void
+cleanup_client()
+{
+  BlockHeapDestroy(client_heap);
+}
+
 struct Client *
 make_client(struct Client *from)
 {
@@ -225,6 +231,8 @@ close_connection(struct Client *client_p)
   client_p->from = NULL; /* ...this should catch them! >:) --msa */
 
   ilog(L_DEBUG, "Closed connection to %s", client_p->name);
+  if(client_p == me.uplink)
+    me.uplink = NULL;
 }
 
 /*
