@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: dbm.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: dbm.c 1083 2007-06-25 21:55:10Z tjfontaine $
  */
 
 #include "stdinc.h"
@@ -235,8 +235,6 @@ init_db()
   Database.yada = yada_init(dbstr, 0);
   MyFree(dbstr);
 
-  MyFree(dbstr);
-
   snprintf(logpath, LOG_BUFSIZE, "%s/%s", LOGDIR, Logging.sqllog);
   if(db_log_fb == NULL)
   {
@@ -1274,7 +1272,7 @@ struct RegChannel *
 db_find_chan(const char *channel)
 {
   yada_rc_t *rc, *brc;
-  struct RegChannel *channel_p;
+  struct RegChannel *channel_p = NULL;
   char *retchan;
 
   assert(channel != NULL);
@@ -1299,6 +1297,7 @@ db_find_chan(const char *channel)
     db_log("db_find_chan: '%s' not found.\n", channel);
     Free(brc);
     Free(rc);
+    free_regchan(channel_p);
     return NULL;
   }
 
@@ -1311,7 +1310,7 @@ db_find_chan(const char *channel)
   DupString(channel_p->topic, channel_p->topic);
   DupString(channel_p->mlock, channel_p->mlock);
 
-  db_log("db_find_chan: Found nick %s\n", channel_p->channel);
+  db_log("db_find_chan: Found chan %s\n", channel_p->channel);
 
   Free(brc);
   Free(rc);
@@ -1388,7 +1387,10 @@ db_find_chanaccess(unsigned int channel, unsigned int account)
       &access->level);
 
   if(Fetch(rc, brc) == 0)
+  {
+    MyFree(access);
     access = NULL;
+  }
 
   Free(rc);
   Free(brc);
