@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: interface.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: interface.c 1049 2007-05-23 16:27:48Z swalsh $
  */
 
 #include "stdinc.h"
@@ -1331,15 +1331,6 @@ free_serviceban(struct ServiceBan *ban)
   MyFree(ban);
 }
 
-void
-free_chanaccess(struct ChanAccess *access)
-{
-  if(access == NULL)
-    return;
-	ilog(L_DEBUG, "Freeing chanaccess %p for channel %d", access, access->channel);
-	MyFree(access);
-}
-
 int 
 check_nick_pass(struct Nick *nick, const char *password)
 {
