@@ -23,7 +23,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: parse.c 1043 2007-05-18 09:24:56Z swalsh $
+ *  $Id: parse.c 1049 2007-05-23 16:27:48Z swalsh $
  */
 
 #include "stdinc.h"
@@ -397,7 +397,7 @@ handle_services_command(struct ServiceMessage *pmptr,
       else
       {
         level = access->level;
-        free_chanaccess(access);
+        MyFree(access);
       }
 
       if(from->access == SUDO_FLAG)
@@ -412,7 +412,7 @@ handle_services_command(struct ServiceMessage *pmptr,
       else
         reply_user(service, NULL, from, SERV_NO_ACCESS_CHAN_ID, mptr->cmd,
             regchptr->channel);
-      if(regchptr != chptr->regchan)
+      if(chptr == NULL || regchptr != chptr->regchan)
         free_regchan(regchptr);
       return;
     }
